@@ -3,7 +3,7 @@
 
 # To enable Gnocchi service, add the following to localrc:
 #
-#   enable_plugin gnocchi https://github.com/openstack/gnocchi master
+#   enable_plugin gnocchi https://github.com/gnocchixyz/gnocchi master
 #
 # This will turn on both gnocchi-api and gnocchi-metricd services.
 # If you don't want one of those (you do) you can use the
@@ -33,11 +33,6 @@ XTRACE=$(set +o | grep xtrace)
 set -o xtrace
 
 
-# Defaults
-# --------
-GITDIR["python-gnocchiclient"]=$DEST/python-gnocchiclient
-GITREPO["python-gnocchiclient"]=${GNOCCHICLIENT_REPO:-${GIT_BASE}/openstack/python-gnocchiclient.git}
-
 if [ -z "$GNOCCHI_DEPLOY" ]; then
     # Default
     GNOCCHI_DEPLOY=simple
@@ -45,13 +40,6 @@ if [ -z "$GNOCCHI_DEPLOY" ]; then
     # Fallback to common wsgi devstack configuration
     if [ "$ENABLE_HTTPD_MOD_WSGI_SERVICES" == "True" ]; then
         GNOCCHI_DEPLOY=mod_wsgi
-
-    # Deprecated config
-    elif [ -n "$GNOCCHI_USE_MOD_WSGI" ] ; then
-        echo_summary "GNOCCHI_USE_MOD_WSGI is deprecated, use GNOCCHI_DEPLOY instead"
-        if [ "$GNOCCHI_USE_MOD_WSGI" == True ]; then
-            GNOCCHI_DEPLOY=mod_wsgi
-        fi
     fi
 fi
 
@@ -116,15 +104,6 @@ function gnocchi_service_url {
     fi
 }
 
-function install_gnocchiclient {
-    if use_library_from_git python-gnocchiclient; then
-        git_clone_by_name python-gnocchiclient
-        setup_dev_lib python-gnocchiclient
-    else
-        pip_install gnocchiclient
-    fi
-}
-
 # install redis
 # NOTE(chdent): We shouldn't rely on ceilometer being present so cannot
 # use its install_redis. There are enough packages now using redis
@@ -152,11 +131,13 @@ function _gnocchi_install_grafana {
     elif is_fedora; then
         sudo yum install "$GRAFANA_RPM_PKG"
     fi
+    sudo -u grafana mkdir -p /var/lib/grafana/plugins
+    sudo rm -rf /var/lib/grafana/plugins/grafana-gnocchi-datasource
     if [ ! "$GRAFANA_PLUGIN_VERSION" ]; then
-        sudo grafana-cli plugins install sileht-gnocchi-datasource
+        sudo grafana-cli plugins install gnocchixyz-gnocchi-datasource
     elif [ "$GRAFANA_PLUGIN_VERSION" != "git" ]; then
-        tmpfile=/tmp/sileht-gnocchi-datasource-${GRAFANA_PLUGIN_VERSION}.tar.gz
-        wget https://github.com/sileht/grafana-gnocchi-datasource/releases/download/${GRAFANA_PLUGIN_VERSION}/sileht-gnocchi-datasource-${GRAFANA_PLUGIN_VERSION}.tar.gz -O $tmpfile
+        tmpfile=/tmp/gnocchixyz-gnocchi-datasource-${GRAFANA_PLUGIN_VERSION}.tar.gz
+        wget https://github.com/gnocchixyz/grafana-gnocchi-datasource/releases/download/${GRAFANA_PLUGIN_VERSION}/gnocchixyz-gnocchi-datasource-${GRAFANA_PLUGIN_VERSION}.tar.gz -O $tmpfile
         sudo -u grafana tar -xzf $tmpfile -C /var/lib/grafana/plugins
         rm -f $file
     else
@@ -169,14 +150,11 @@ function _gnocchi_install_grafana {
 }
 
 function _cleanup_gnocchi_apache_wsgi {
-    sudo rm -f $GNOCCHI_WSGI_DIR/*.wsgi
     sudo rm -f $(apache_site_config_for gnocchi)
 }
 
 # _config_gnocchi_apache_wsgi() - Set WSGI config files of Gnocchi
 function _config_gnocchi_apache_wsgi {
-    sudo mkdir -p $GNOCCHI_WSGI_DIR
-
     local gnocchi_apache_conf=$(apache_site_config_for gnocchi)
     local venv_path=""
     local script_name=$GNOCCHI_SERVICE_PREFIX
@@ -185,9 +163,6 @@ function _config_gnocchi_apache_wsgi {
         venv_path="python-path=${PROJECT_VENV["gnocchi"]}/lib/$(python_version)/site-packages"
     fi
 
-    # copy wsgi file
-    sudo cp $GNOCCHI_DIR/gnocchi/rest/app.wsgi $GNOCCHI_WSGI_DIR/
-
     # Only run the API on a custom PORT if it has been specifically
     # asked for.
     if [[ -n $GNOCCHI_SERVICE_PORT ]]; then
@@ -203,7 +178,7 @@ function _config_gnocchi_apache_wsgi {
     fi
     sudo sed -e "
             s|%APACHE_NAME%|$APACHE_NAME|g;
-            s|%WSGI%|$GNOCCHI_WSGI_DIR/app.wsgi|g;
+            s|%WSGI%|$GNOCCHI_BIN_DIR/gnocchi-api|g;
             s|%USER%|$STACK_USER|g
             s|%APIWORKERS%|$API_WORKERS|g
             s|%VIRTUALENV%|$venv_path|g
@@ -240,13 +215,12 @@ function configure_gnocchi {
     fi
 
     if [ -n "$GNOCCHI_COORDINATOR_URL" ]; then
-        iniset $GNOCCHI_CONF storage coordination_url "$GNOCCHI_COORDINATOR_URL"
+        iniset $GNOCCHI_CONF coordination_url "$GNOCCHI_COORDINATOR_URL"
     fi
 
     if is_service_enabled gnocchi-statsd ; then
         iniset $GNOCCHI_CONF statsd resource_id $GNOCCHI_STATSD_RESOURCE_ID
-        iniset $GNOCCHI_CONF statsd project_id $GNOCCHI_STATSD_PROJECT_ID
-        iniset $GNOCCHI_CONF statsd user_id $GNOCCHI_STATSD_USER_ID
+        iniset $GNOCCHI_CONF statsd creator $GNOCCHI_STATSD_CREATOR
     fi
 
     # Configure the storage driver
@@ -264,6 +238,9 @@ function configure_gnocchi {
     elif [[ "$GNOCCHI_STORAGE_BACKEND" = 'file' ]] ; then
         iniset $GNOCCHI_CONF storage driver file
         iniset $GNOCCHI_CONF storage file_basepath $GNOCCHI_DATA_DIR/
+    elif [[ "$GNOCCHI_STORAGE_BACKEND" = 'redis' ]] ; then
+        iniset $GNOCCHI_CONF storage driver redis
+        iniset $GNOCCHI_CONF storage redis_url $GNOCCHI_REDIS_URL
     else
         echo "ERROR: could not configure storage driver"
         exit 1
@@ -292,7 +269,7 @@ function configure_gnocchi {
         rm -f "$GNOCCHI_UWSGI_FILE"
 
         iniset "$GNOCCHI_UWSGI_FILE" uwsgi http $GNOCCHI_SERVICE_HOST:$GNOCCHI_SERVICE_PORT
-        iniset "$GNOCCHI_UWSGI_FILE" uwsgi wsgi-file "$GNOCCHI_DIR/gnocchi/rest/app.wsgi"
+        iniset "$GNOCCHI_UWSGI_FILE" uwsgi wsgi-file "$GNOCCHI_BIN_DIR/gnocchi-api"
         # This is running standalone
         iniset "$GNOCCHI_UWSGI_FILE" uwsgi master true
         # Set die-on-term & exit-on-reload so that uwsgi shuts down
@@ -348,11 +325,7 @@ function init_gnocchi {
     if is_service_enabled mysql postgresql; then
         recreate_database gnocchi
     fi
-    if is_service_enabled ceilometer; then
-        $GNOCCHI_BIN_DIR/gnocchi-upgrade --create-legacy-resource-types
-    else
-        $GNOCCHI_BIN_DIR/gnocchi-upgrade
-    fi
+    $GNOCCHI_BIN_DIR/gnocchi-upgrade
 }
 
 function preinstall_gnocchi {
@@ -371,7 +344,7 @@ function preinstall_gnocchi {
 
 # install_gnocchi() - Collect source and prepare
 function install_gnocchi {
-    if [ "${GNOCCHI_COORDINATOR_URL%%:*}" == "redis" ]; then
+    if [[ "$GNOCCHI_STORAGE_BACKEND" = 'redis' ]] || [[ "${GNOCCHI_COORDINATOR_URL%%:*}" == "redis" ]]; then
         _gnocchi_install_redis
     fi
 
@@ -384,8 +357,6 @@ function install_gnocchi {
         _gnocchi_install_grafana
     fi
 
-    install_gnocchiclient
-
     [ "$GNOCCHI_USE_KEYSTONE" == "True" ] && EXTRA_FLAVOR=,keystone
 
     # We don't use setup_package because we don't follow openstack/requirements
@@ -434,8 +405,8 @@ function start_gnocchi {
     fi
 
     # run metricd last so we are properly waiting for swift and friends
-    run_process gnocchi-metricd "$GNOCCHI_BIN_DIR/gnocchi-metricd -d -v --config-file $GNOCCHI_CONF"
-    run_process gnocchi-statsd "$GNOCCHI_BIN_DIR/gnocchi-statsd -d -v --config-file $GNOCCHI_CONF"
+    run_process gnocchi-metricd "$GNOCCHI_BIN_DIR/gnocchi-metricd -d --config-file $GNOCCHI_CONF"
+    run_process gnocchi-statsd "$GNOCCHI_BIN_DIR/gnocchi-statsd -d --config-file $GNOCCHI_CONF"
 }
 
 # stop_gnocchi() - Stop running processes
