@@ -14,6 +14,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 from __future__ import absolute_import
+import datetime
 import itertools
 import operator
 import os.path
@@ -22,11 +23,11 @@ import uuid
 
 from alembic import migration
 from alembic import operations
+import daiquiri
 import oslo_db.api
 from oslo_db import exception
 from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import utils as oslo_db_utils
-from oslo_log import log
 try:
     import psycopg2
 except ImportError:
@@ -46,6 +47,7 @@ import sqlalchemy_utils
 from gnocchi import exceptions
 from gnocchi import indexer
 from gnocchi.indexer import sqlalchemy_base as base
+from gnocchi import resource_type
 from gnocchi import utils
 
 Base = base.Base
@@ -58,18 +60,32 @@ ResourceType = base.ResourceType
 
 _marker = indexer._marker
 
-LOG = log.getLogger(__name__)
+LOG = daiquiri.getLogger(__name__)
 
 
 def _retry_on_exceptions(exc):
-    return (
+    if not isinstance(exc, exception.DBError):
+        return False
+    inn_e = exc.inner_exception
+    if not isinstance(inn_e, sqlalchemy.exc.InternalError):
+        return False
+    return ((
         pymysql and
-        isinstance(exc, exception.DBError) and
-        isinstance(exc.inner_exception, sqlalchemy.exc.InternalError) and
-        isinstance(exc.inner_exception.orig, pymysql.err.InternalError) and
-        (exc.inner_exception.orig.args[0] ==
-         pymysql.constants.ER.TABLE_DEF_CHANGED)
-    )
+        isinstance(inn_e.orig, pymysql.err.InternalError) and
+        (inn_e.orig.args[0] == pymysql.constants.ER.TABLE_DEF_CHANGED)
+    ) or (
+        # HACK(jd) Sometimes, PostgreSQL raises an error such as "current
+        # transaction is aborted, commands ignored until end of transaction
+        # block" on its own catalog, so we need to retry, but this is not
+        # caught by oslo.db as a deadlock. This is likely because when we use
+        # Base.metadata.create_all(), sqlalchemy itself gets an error it does
+        # not catch or something. So this is why this function exists. To
+        # paperover I guess.
+        psycopg2
+        and isinstance(inn_e.orig, psycopg2.InternalError)
+        # current transaction is aborted
+        and inn_e.orig.pgcode == '25P02'
+    ))
 
 
 def retry_on_deadlock(f):
@@ -176,34 +192,13 @@ class ResourceClassMapper(object):
         tables = [Base.metadata.tables[mappers["resource"].__tablename__],
                   Base.metadata.tables[mappers["history"].__tablename__]]
 
-        try:
-            with facade.writer_connection() as connection:
-                Base.metadata.create_all(connection, tables=tables)
-        except exception.DBError as e:
-            if self._is_current_transaction_aborted(e):
-                raise exception.RetryRequest(e)
-            raise
+        with facade.writer_connection() as connection:
+            Base.metadata.create_all(connection, tables=tables)
 
         # NOTE(sileht): no need to protect the _cache with a lock
         # get_classes cannot be called in state creating
         self._cache[resource_type.tablename] = mappers
 
-    @staticmethod
-    def _is_current_transaction_aborted(exception):
-        # HACK(jd) Sometimes, PostgreSQL raises an error such as "current
-        # transaction is aborted, commands ignored until end of transaction
-        # block" on its own catalog, so we need to retry, but this is not
-        # caught by oslo.db as a deadlock. This is likely because when we use
-        # Base.metadata.create_all(), sqlalchemy itself gets an error it does
-        # not catch or something. So this is why this function exists. To
-        # paperover I guess.
-        inn_e = exception.inner_exception
-        return (psycopg2
-                and isinstance(inn_e, sqlalchemy.exc.InternalError)
-                and isinstance(inn_e.orig, psycopg2.InternalError)
-                # current transaction is aborted
-                and inn_e.orig.pgcode == '25P02')
-
     @retry_on_deadlock
     def unmap_and_delete_tables(self, resource_type, facade):
         if resource_type.state != "deleting":
@@ -225,25 +220,20 @@ class ResourceClassMapper(object):
         # so this code cannot be triggerred anymore for this
         # resource_type
         with facade.writer_connection() as connection:
-            try:
-                for table in tables:
-                    for fk in table.foreign_key_constraints:
-                        try:
-                            self._safe_execute(
-                                connection,
-                                sqlalchemy.schema.DropConstraint(fk))
-                        except exception.DBNonExistentConstraint:
-                            pass
-                for table in tables:
+            for table in tables:
+                for fk in table.foreign_key_constraints:
                     try:
-                        self._safe_execute(connection,
-                                           sqlalchemy.schema.DropTable(table))
-                    except exception.DBNonExistentTable:
+                        self._safe_execute(
+                            connection,
+                            sqlalchemy.schema.DropConstraint(fk))
+                    except exception.DBNonExistentConstraint:
                         pass
-            except exception.DBError as e:
-                if self._is_current_transaction_aborted(e):
-                    raise exception.RetryRequest(e)
-                raise
+            for table in tables:
+                try:
+                    self._safe_execute(connection,
+                                       sqlalchemy.schema.DropTable(table))
+                except exception.DBNonExistentTable:
+                    pass
 
             # NOTE(sileht): If something goes wrong here, we are currently
             # fucked, that why we expose the state to the superuser.
@@ -286,6 +276,10 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             url = sqlalchemy_url.make_url(url)
             url.drivername = "mysql+pymysql"
             return str(url)
+        if url.startswith("postgresql://"):
+            url = sqlalchemy_url.make_url(url)
+            url.drivername = "postgresql+psycopg2"
+            return str(url)
         return url
 
     def __init__(self, conf):
@@ -295,6 +289,9 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         self.conf = conf
         self.facade = PerInstanceFacade(conf)
 
+    def __str__(self):
+        return "%s: %s" % (self.__class__.__name__, self.conf.indexer.url)
+
     def disconnect(self):
         self.facade.dispose()
 
@@ -304,13 +301,13 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         cfg = config.Config(
             "%s/alembic/alembic.ini" % os.path.dirname(__file__))
         cfg.set_main_option('sqlalchemy.url',
-                            self.conf.database.connection.replace('%', '%%'))
+                            self.conf.database.connection)
         return cfg
 
     def get_engine(self):
         return self.facade.get_engine()
 
-    def upgrade(self, nocreate=False, create_legacy_resource_types=False):
+    def upgrade(self, nocreate=False):
         from alembic import command
         from alembic import migration
 
@@ -328,30 +325,16 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                 else:
                     command.upgrade(cfg, "head")
 
-        # TODO(sileht): generic shouldn't be a particular case
-        # we must create a rt_generic and rt_generic_history table
-        # like other type
-        for rt in base.get_legacy_resource_types():
-            if not (rt.name == "generic" or create_legacy_resource_types):
-                continue
-
-            try:
-                with self.facade.writer() as session:
-                    session.add(rt)
-            except exception.DBDuplicateEntry:
-                continue
-
-            if rt.name != "generic":
-                try:
-                    self._RESOURCE_TYPE_MANAGER.map_and_create_tables(
-                        rt, self.facade)
-                except Exception:
-                    self._set_resource_type_state(rt.name, "creation_error")
-                    LOG.exception('Fail to create tables for '
-                                  'resource_type "%s"', rt.name)
-                    continue
-
-            self._set_resource_type_state(rt.name, "active")
+        try:
+            with self.facade.writer() as session:
+                session.add(
+                    ResourceType(
+                        name="generic",
+                        tablename="generic",
+                        state="active",
+                        attributes=resource_type.ResourceTypeAttributes()))
+        except exception.DBDuplicateEntry:
+            pass
 
     # NOTE(jd) We can have deadlock errors either here or later in
     # map_and_create_tables(). We can't decorate create_resource_type()
@@ -600,7 +583,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                 if c.granularity != n.granularity:
                     raise indexer.UnsupportedArchivePolicyChange(
                         name, '%s granularity interval was changed'
-                        % c.granularity)
+                        % utils.timespan_total_seconds(c.granularity))
             # NOTE(gordc): ORM doesn't update JSON column unless new
             ap.definition = ap_items
             return ap
@@ -636,7 +619,9 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
     def list_archive_policy_rules(self):
         with self.facade.independent_reader() as session:
             return session.query(ArchivePolicyRule).order_by(
-                ArchivePolicyRule.metric_pattern.desc()).all()
+                ArchivePolicyRule.metric_pattern.desc(),
+                ArchivePolicyRule.name.asc()
+            ).all()
 
     def get_archive_policy_rule(self, name):
         with self.facade.independent_reader() as session:
@@ -662,6 +647,21 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             raise indexer.ArchivePolicyRuleAlreadyExists(name)
         return apr
 
+    def update_archive_policy_rule(self, name, new_name):
+        apr = self.get_archive_policy_rule(name)
+        if not apr:
+            raise indexer.NoSuchArchivePolicyRule(name)
+        apr.name = new_name
+        try:
+            with self.facade.writer() as session:
+                session.add(apr)
+        except exception.DBDuplicateEntry:
+            raise indexer.UnsupportedArchivePolicyRuleChange(
+                name,
+                'Archive policy rule %s already exists.'
+                % new_name)
+        return apr
+
     @retry_on_deadlock
     def create_metric(self, id, creator, archive_policy_name,
                       name=None, unit=None, resource_id=None):
@@ -688,7 +688,7 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
     @retry_on_deadlock
     def list_metrics(self, names=None, ids=None, details=False,
                      status='active', limit=None, marker=None, sorts=None,
-                     **kwargs):
+                     creator=None, **kwargs):
         sorts = sorts or []
         if ids is not None and not ids:
             return []
@@ -701,12 +701,19 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
                 q = q.filter(Metric.name.in_(names))
             if ids is not None:
                 q = q.filter(Metric.id.in_(ids))
+            if creator is not None:
+                if creator[0] == ":":
+                    q = q.filter(Metric.creator.like("%%%s" % creator))
+                elif creator[-1] == ":":
+                    q = q.filter(Metric.creator.like("%s%%" % creator))
+                else:
+                    q = q.filter(Metric.creator == creator)
             for attr in kwargs:
                 q = q.filter(getattr(Metric, attr) == kwargs[attr])
             if details:
                 q = q.options(sqlalchemy.orm.joinedload('resource'))
 
-            sort_keys, sort_dirs = self._build_sort_keys(sorts)
+            sort_keys, sort_dirs = self._build_sort_keys(sorts, ['id'])
 
             if marker:
                 metric_marker = self.list_metrics(ids=[marker])
@@ -991,9 +998,11 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             if history:
                 target_cls = self._get_history_result_mapper(
                     session, resource_type)
+                unique_keys = ["id", "revision"]
             else:
                 target_cls = self._resource_type_to_mappers(
                     session, resource_type)["resource"]
+                unique_keys = ["id"]
 
             q = session.query(target_cls)
 
@@ -1011,10 +1020,24 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
 
                 q = q.filter(f)
 
-            sort_keys, sort_dirs = self._build_sort_keys(sorts)
+            sort_keys, sort_dirs = self._build_sort_keys(sorts, unique_keys)
 
             if marker:
-                resource_marker = self.get_resource(resource_type, marker)
+                marker_q = session.query(target_cls)
+                if history:
+                    try:
+                        rid, rrev = marker.split("@")
+                        rrev = int(rrev)
+                    except ValueError:
+                        resource_marker = None
+                    else:
+                        resource_marker = marker_q.filter(
+                            target_cls.id == rid,
+                            target_cls.revision == rrev).first()
+                else:
+                    resource_marker = marker_q.filter(
+                        target_cls.id == marker).first()
+
                 if resource_marker is None:
                     raise indexer.InvalidPagination(
                         "Invalid marker: `%s'" % marker)
@@ -1092,11 +1115,11 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
         with self.facade.writer() as session:
             if session.query(Metric).filter(
                 Metric.id == id, Metric.status == 'active').update(
-                    {"status": "delete", "resource_id": None}) == 0:
+                    {"status": "delete"}) == 0:
                 raise indexer.NoSuchMetric(id)
 
     @staticmethod
-    def _build_sort_keys(sorts):
+    def _build_sort_keys(sorts, unique_keys):
         # transform the api-wg representation to the oslo.db one
         sort_keys = []
         sort_dirs = []
@@ -1106,21 +1129,16 @@ class SQLAlchemyIndexer(indexer.IndexerDriver):
             sort_dirs.append(sort_dir or 'asc')
 
         # paginate_query require at list one uniq column
-        if 'id' not in sort_keys:
-            sort_keys.append('id')
-            sort_dirs.append('asc')
+        for key in unique_keys:
+            if key not in sort_keys:
+                sort_keys.append(key)
+                sort_dirs.append('asc')
 
         return sort_keys, sort_dirs
 
 
-def _operator_in(field_name, value):
-    # Do not generate empty IN comparison
-    # https://github.com/gnocchixyz/gnocchi/issues/530
-    if len(value):
-        return field_name.in_(value)
-
-
 class QueryTransformer(object):
+
     unary_operators = {
         u"not": sqlalchemy.not_,
     }
@@ -1148,7 +1166,7 @@ class QueryTransformer(object):
         u"≠": operator.ne,
         u"ne": operator.ne,
 
-        u"in": _operator_in,
+        u"in": lambda field_name, values: field_name.in_(values),
 
         u"like": lambda field, value: field.like(value),
     }
@@ -1188,7 +1206,9 @@ class QueryTransformer(object):
 
         if field_name == "lifespan":
             attr = getattr(table, "ended_at") - getattr(table, "started_at")
-            value = utils.to_timespan(value)
+            value = datetime.timedelta(
+                seconds=utils.timespan_total_seconds(
+                    utils.to_timespan(value)))
             if engine == "mysql":
                 # NOTE(jd) So subtracting 2 timestamps in MySQL result in some
                 # weird results based on string comparison. It's useless and it
@@ -1259,6 +1279,6 @@ class QueryTransformer(object):
                     op = cls.unary_operators[operator]
                 except KeyError:
                     raise indexer.QueryInvalidOperator(operator)
-                return cls._handle_unary_op(engine, table, op, nodes)
+                return cls._handle_unary_op(engine, op, nodes)
             return cls._handle_binary_op(engine, table, op, nodes)
         return cls._handle_multiple_op(engine, table, op, nodes)
