@@ -19,25 +19,24 @@ try:
     import asyncio
 except ImportError:
     import trollius as asyncio
+import daiquiri
 from oslo_config import cfg
-from oslo_log import log
 import six
 
+from gnocchi import incoming
 from gnocchi import indexer
 from gnocchi import service
-from gnocchi import storage
 from gnocchi import utils
 
 
-LOG = log.getLogger(__name__)
+LOG = daiquiri.getLogger(__name__)
 
 
 class Stats(object):
     def __init__(self, conf):
         self.conf = conf
-        self.storage = storage.get_driver(self.conf)
+        self.incoming = incoming.get_driver(self.conf)
         self.indexer = indexer.get_driver(self.conf)
-        self.indexer.connect()
         try:
             self.indexer.create_resource('generic',
                                          self.conf.statsd.resource_id,
@@ -50,6 +49,13 @@ class Stats(object):
         self.gauges = {}
         self.counters = {}
         self.times = {}
+        self.metrics = {
+            metric.name: metric
+            for metric
+            in self.indexer.get_resource('generic',
+                                         self.conf.statsd.resource_id,
+                                         with_metrics=True).metrics
+        }
 
     def reset(self):
         self.gauges.clear()
@@ -63,14 +69,14 @@ class Stats(object):
                 raise ValueError(
                     "Invalid sampling for ms: `%d`, should be none"
                     % sampling)
-            self.times[metric_name] = storage.Measure(
+            self.times[metric_name] = incoming.Measure(
                 utils.dt_in_unix_ns(utils.utcnow()), value)
         elif metric_type == "g":
             if sampling is not None:
                 raise ValueError(
                     "Invalid sampling for g: `%d`, should be none"
                     % sampling)
-            self.gauges[metric_name] = storage.Measure(
+            self.gauges[metric_name] = incoming.Measure(
                 utils.dt_in_unix_ns(utils.utcnow()), value)
         elif metric_type == "c":
             sampling = 1 if sampling is None else sampling
@@ -78,7 +84,7 @@ class Stats(object):
                 current_value = self.counters[metric_name].value
             else:
                 current_value = 0
-            self.counters[metric_name] = storage.Measure(
+            self.counters[metric_name] = incoming.Measure(
                 utils.dt_in_unix_ns(utils.utcnow()),
                 current_value + (value * (1 / sampling)))
         # TODO(jd) Support "set" type
@@ -88,10 +94,6 @@ class Stats(object):
             raise ValueError("Unknown metric type `%s'" % metric_type)
 
     def flush(self):
-        resource = self.indexer.get_resource('generic',
-                                             self.conf.statsd.resource_id,
-                                             with_metrics=True)
-
         for metric_name, measure in itertools.chain(
                 six.iteritems(self.gauges),
                 six.iteritems(self.counters),
@@ -101,7 +103,7 @@ class Stats(object):
                 # is not designed to run in parallel and we do not envision
                 # operators manipulating the resource/metrics using the Gnocchi
                 # API at the same time.
-                metric = resource.get_metric(metric_name)
+                metric = self.metrics.get(metric_name)
                 if not metric:
                     ap_name = self._get_archive_policy_name(metric_name)
                     metric = self.indexer.create_metric(
@@ -110,7 +112,8 @@ class Stats(object):
                         archive_policy_name=ap_name,
                         name=metric_name,
                         resource_id=self.conf.statsd.resource_id)
-                self.storage.incoming.add_measures(metric, (measure,))
+                    self.metrics[metric_name] = metric
+                self.incoming.add_measures(metric, (measure,))
             except Exception as e:
                 LOG.error("Unable to add measure %s: %s",
                           metric_name, e)
@@ -125,7 +128,7 @@ class Stats(object):
         return ap.name
 
 
-class StatsdServer(object):
+class StatsdServer(asyncio.Protocol):
     def __init__(self, stats):
         self.stats = stats
 
