@@ -41,6 +41,9 @@ class TestUtils(tests_base.TestCase):
                                tzinfo=iso8601.iso8601.FixedOffset(5, 0, '+5h'))
         self._do_test_datetime_to_unix_timezone_change(1420106400.0, dt)
 
+    def test_to_timestamp_empty(self):
+        self.assertEqual([], utils.to_timestamps([]))
+
     def test_to_timestamps_epoch(self):
         self.assertEqual(
             utils.to_datetime("1425652440"),
@@ -59,6 +62,15 @@ class TestUtils(tests_base.TestCase):
             datetime.datetime(2015, 3, 6, 14, 34, 0, 400000,
                               tzinfo=iso8601.iso8601.UTC))
 
+    def test_to_timestamps_relative(self):
+        with mock.patch('gnocchi.utils.utcnow') as utcnow:
+            utcnow.return_value = datetime.datetime(
+                2015, 3, 6, 14, 34, tzinfo=iso8601.iso8601.UTC)
+            self.assertEqual(
+                utils.to_datetime("-10 minutes"),
+                datetime.datetime(2015, 3, 6, 14, 24,
+                                  tzinfo=iso8601.iso8601.UTC))
+
 
 class TestResourceUUID(tests_base.TestCase):
     def test_conversion(self):
@@ -76,3 +88,44 @@ class TestResourceUUID(tests_base.TestCase):
         self.assertEqual(
             uuid.UUID('853e5c64-f45e-58b2-999c-96df856fbe3d'),
             utils.ResourceUUID("foo", ""))
+
+
+class StopWatchTest(tests_base.TestCase):
+    def test_no_states(self):
+        watch = utils.StopWatch()
+        self.assertRaises(RuntimeError, watch.stop)
+
+    def test_start_stop(self):
+        watch = utils.StopWatch()
+        watch.start()
+        watch.stop()
+
+    def test_no_elapsed(self):
+        watch = utils.StopWatch()
+        self.assertRaises(RuntimeError, watch.elapsed)
+
+    def test_elapsed(self):
+        watch = utils.StopWatch()
+        watch.start()
+        watch.stop()
+        elapsed = watch.elapsed()
+        self.assertAlmostEqual(elapsed, watch.elapsed())
+
+    def test_context_manager(self):
+        with utils.StopWatch() as watch:
+            pass
+        self.assertGreater(watch.elapsed(), 0)
+
+
+class ParallelMap(tests_base.TestCase):
+    def test_parallel_map_one(self):
+        utils.parallel_map.NUM_WORKERS = 1
+        self.assertEqual([1, 2, 3],
+                         utils.parallel_map(lambda x: x,
+                                            [[1], [2], [3]]))
+
+    def test_parallel_map_four(self):
+        utils.parallel_map.NUM_WORKERS = 4
+        self.assertEqual([1, 2, 3],
+                         utils.parallel_map(lambda x: x,
+                                            [[1], [2], [3]]))
